// XMP2 (eXperimental Media Player #2).java

import java.awt.AlphaComposite;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.geom.Rectangle2D;

import java.net.URI;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sun.media.jmc.MediaProvider;

import com.sun.media.jmc.control.AudioControl;
import com.sun.media.jmc.control.VideoRenderControl;

import com.sun.media.jmc.event.VideoRendererEvent;
import com.sun.media.jmc.event.VideoRendererListener;

public class XMP2 extends JFrame
{
   MediaProvider mp;

   public XMP2 (String mediaURI)
   {
      super ("XMP2: "+mediaURI);
      setDefaultCloseOperation (EXIT_ON_CLOSE);

      try
      {
          mp = new MediaProvider (new URI (mediaURI));
      }
      catch (Exception e)
      {
          System.out.println ("Error opening media: "+e.toString ());
          System.exit (1);
      }

      JPanel panel = new JPanel ();
      panel.setLayout (new BorderLayout ());
      panel.add (new MediaPanel (mp), BorderLayout.CENTER);
      panel.add (createControlPanel (), BorderLayout.SOUTH);

      setContentPane (panel);

      pack ();
      setVisible (true);
   }

   public JPanel createControlPanel ()
   {
      JPanel panel = new JPanel ();

      JButton btnPlay = new JButton ("Play");
      ActionListener alPlay = new ActionListener ()
                              {
                                  public void actionPerformed (ActionEvent ae)
                                  {
                                     mp.play ();
                                  }
                              };
      btnPlay.addActionListener (alPlay);
      panel.add (btnPlay);

      JButton btnPause = new JButton ("Pause");
      ActionListener alPause = new ActionListener ()
                               {
                                   public void actionPerformed (ActionEvent ae)
                                   {
                                      mp.pause ();
                                   }
                               };
      btnPause.addActionListener (alPause);
      panel.add (btnPause);

      JButton btnStop = new JButton ("Stop");
      ActionListener alStop = new ActionListener ()
                              {
                                  public void actionPerformed (ActionEvent ae)
                                  {
                                     mp.pause ();
                                     mp.setMediaTime (0.0);
                                  }
                              };
      btnStop.addActionListener (alStop);
      panel.add (btnStop);

      JCheckBox cbMute = new JCheckBox ("Mute");
      ChangeListener clMute = new ChangeListener ()
                              {
                                  public void stateChanged (ChangeEvent ce)
                                  {
                                     JCheckBox cb;
                                     cb = (JCheckBox) ce.getSource ();
                                     AudioControl ac;
                                     ac = mp.getControl (AudioControl.class);
                                     ac.setMute (cb.isSelected ());
                                  }
                              };
      cbMute.addChangeListener (clMute);
      panel.add (cbMute);

      panel.add (new JLabel ("Duration: "+mp.getDuration ()));

      return panel;
   }

   public static void main (final String [] args)
   {
      if (args.length != 1)
      {
          System.err.println ("usage: java XMP2 mediaURI");
          return;
      }

      Runnable r = new Runnable ()
                   {
                       public void run ()
                       {
                          new XMP2 (args [0]);
                       }
                   };
      EventQueue.invokeLater (r);
   }
}

class MediaPanel extends JPanel
{
   private AlphaComposite ac1 =
     AlphaComposite.getInstance (AlphaComposite.SRC_OVER, 0.1f);

   private AlphaComposite ac2 =
     AlphaComposite.getInstance (AlphaComposite.SRC_OVER, 0.3f);

   private Dimension frameSize;

   private Font font = new Font ("Arial", Font.BOLD, 16);

   private Rectangle frame = new Rectangle (0, 0, 0, 0);

   private Rectangle2D r2d = new Rectangle2D.Double (0.0, 0.0, 0.0, 0.0);

   private VideoRenderControl vrc;

   MediaPanel (MediaProvider mp)
   {
      vrc = mp.getControl (VideoRenderControl.class);
      if (vrc == null)
      {
          System.err.println ("no video renderer control");
          System.exit (-1);
      }
      frameSize = vrc.getFrameSize ();
      VideoRendererListener vrl;
      vrl = new VideoRendererListener ()
            {
                public void videoFrameUpdated (VideoRendererEvent vre)
                {
                   repaint ();
                }
            };
      vrc.addVideoRendererListener (vrl);
      setPreferredSize (new Dimension (frameSize.width/2, frameSize.height/2));
   }

   protected void paintComponent (Graphics g)
   {
      frame.width = getWidth ();
      frame.height = getHeight ();
      vrc.paintVideoFrame (g, frame);

      double ar1 = frame.width/(double) frameSize.width;
      double ar2 = frame.height/(double) frameSize.height;

      double x, y;

      if (ar1 < ar2)
      {
          x = (frame.width-frameSize.width*ar1)/2+10.0;
          y = (frame.height-frameSize.height*ar1)/2+10.0;
      }
      else
      {
          x = (frame.width-frameSize.width*ar2)/2+10.0;
          y = (frame.height-frameSize.height*ar2)/2+10.0;
      }
      
      Graphics2D g2d = (Graphics2D) g;
      g2d.setRenderingHint (RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
      g2d.setComposite (ac1);
      g2d.setPaint (Color.black);
      r2d.setFrame (x, y, 125.0, 50.0);
      g2d.fill (r2d);
      g2d.setComposite (ac2);
      g2d.setFont (font);
      g2d.setPaint (Color.yellow);
      g2d.drawString ("javajeff.mb.ca", (int) x+10, (int) y+30);
   }
}
